/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"
#include "consoledefs.h"

typedef struct consolevar_s consolevar_t;

typedef void (*consolevarchanged_t)(const consolevar_t *var);

typedef struct consolevar_s {
  char_t name[CONSOLE_VAR_NAME_MAX + 1];
  char_t value[CONSOLE_VAR_VALUE_MAX + 1];
  consolevarchanged_t events[CONSOLE_VAR_EVENTS_MAX];
  uint8_t eventCount;
} consolevar_t;

/**
 * Initializes a console variable.
 * 
 * @param var Pointer to the console variable.
 * @param name The name of the variable.
 * @param value The initial value of the variable.
 */
void consoleVarInit(
  consolevar_t *var,
  const char_t *name,
  const char_t *value
);

/**
 * Initializes a console variable with a listener.
 * 
 * @param var Pointer to the console variable.
 * @param name The name of the variable.
 * @param value The initial value of the variable.
 * @param event The event to register.
 */
void consoleVarInitListener(
  consolevar_t *var,
  const char_t *name,
  const char_t *value,
  consolevarchanged_t event
);

/**
 * Sets the value of a console variable.
 * 
 * @param var Pointer to the console variable.
 * @param value The new value of the variable.
 */
void consoleVarSetValue(consolevar_t *var, const char_t *value);

/**
 * Registers an event to be called when the value of a console variable changes.
 * 
 * @param var Pointer to the console variable.
 * @param event The event to register.
 */
void consoleVarListen(consolevar_t *var, consolevarchanged_t event);