/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "string.h"
#include "assert/assert.h"

bool_t stringIsWhitespace(const char_t c) {
  return isspace(c);
}

void stringCopy(char_t *dest, const char_t *src, const size_t destSize) {
  assertNotNull(dest, "dest must not be NULL");
  assertNotNull(src, "src must not be NULL");
  assertStrLenMax(src, destSize, "src is too long");
  strncpy(dest, src, destSize);
}

int stringCompare(const char_t *str1, const char_t *str2) {
  assertNotNull(str1, "str1 must not be NULL");
  assertNotNull(str2, "str2 must not be NULL");
  return strcmp(str1, str2);
}

void stringTrim(char_t *str) {
  assertNotNull(str, "str must not be NULL");

  // Trim leading whitespace
  char_t *start = str;
  while(stringIsWhitespace(*start)) start++;

  // Trim trailing whitespace
  char_t *end = start + strlen(start) - 1;
  while (end >= start && stringIsWhitespace(*end)) end--;

  // Null-terminate the string
  *(end + 1) = '\0';

  // Move trimmed string to the original buffer
  if (start != str) memmove(str, start, end - start + 2);
}

char_t * stringToken(char_t *str, const char_t *delim) {
  assertNotNull(str, "str must not be NULL");
  assertNotNull(delim, "delim must not be NULL");
  return strtok(str, delim);
}